<?php
/*
Plugin Name: Active Plugins
Plugin URI: http://trepmal.com/plugins/active-plugins-on-multisite/
Description: Get number of users for each active plugin (minus network-activated). Then break down by site.
Author: Kailey Lampert
Version: 1.5
Author URI: http://kaileylampert.com/
Network: true
*/
/*
    Copyright (C) 2011  Kailey Lampert

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class activeplugins {

	function __construct() {
		add_action('init', array( &$this, 'setup' ) );
	}

	function setup() {
		if ( ! is_multisite() )
			add_action('admin_notices', array( &$this, 'notice' ) );
		else
			add_action('network_admin_menu', array( &$this, 'menu' ) );
	}

	function notice() {
		echo '<div class="error fade"><p>';
		_e( 'Acitve Plugins is for multisite use only.', 'active-plugins' );
		echo '</p></div>';
	}

	function menu() {
		add_submenu_page( 'settings.php', 'Active Plugins', 'Active Plugins', 'unfiltered_html', __FILE__, array( &$this, 'page' ) );
	}

	function page() {

		echo '<div class="wrap">';
		echo '<h2>'; _e( 'Active Plugins', 'active-plugins' ); echo '</h2>';
		echo '<p>'; _e( 'Network-Activated plugins not listed.', 'active-plugins' ); echo '</p>';

		global $wpdb;
		$query =  "SELECT * FROM {$wpdb->blogs}, {$wpdb->registration_log}
							WHERE site_id = '{$wpdb->siteid}'
							AND {$wpdb->blogs}.blog_id = {$wpdb->registration_log}.blog_id";

		$blog_list = $wpdb->get_results( $query, ARRAY_A ); //get blogs
		$all_plugins = get_plugins();
		$plugins_list = array_keys( get_plugins() );
		//echo '<pre>'. print_r( get_plugins(), true) .'</pre>';
		$pi = array();

		//add main site to beginning
		$blog_list[-1] = array( 'blog_id' => 1 );
		ksort($blog_list);
		foreach( $blog_list as $k => $info ) {
			//loop through the blogs
			//store active plugins is giant array index by blog id
			$bid = $info['blog_id'];
			$pi[ $bid ] = get_blog_option( $bid, 'active_plugins' );
		}
		$pi = array_filter($pi); //remove empties

		$pi_count = array();
		foreach($pi as $k => $v_array) {
			//put all active plugins into one array, we can then count duplicate values
			$pi_count = array_merge($pi_count, $v_array);
		}

		echo "<pre style='background:#cfc'>Totals (each active plugin and how many users): \n";
			$totals = $tags = array_count_values( $pi_count );
			ksort( $totals );
			foreach( $totals as $name => $tot) {

				if ( strpos( $name, '/') !== false) {
					$dir = WP_PLUGIN_DIR . '/' . dirname( $name );
					$dottags = ( glob( $dir . '/*.tag') );
					if ( ! empty( $dottags ) )
						$tags[ $name ] = str_replace( $dir . '/', '', str_replace( '.tag', '', $dottags['0'] ) );
				}

				if ( in_array( $name, $plugins_list ) ) {
				 	$plugins_list = array_flip( $plugins_list );
				 	unset( $plugins_list[ $name ] );
				 	$plugins_list = array_flip( $plugins_list );
				}

				$realname = $all_plugins[$name]['Name'] . ' v' . $all_plugins[$name]['Version'];
				//$realname = $all_plugins[$name]['Name'];
				echo "\t<strong>" . $realname . (!is_numeric($tags[ $name ]) ? ' (tagged: '.$tags[ $name ].')' : '') . '</strong> is used by <strong>' . $tot . ' site' . (($tot == '1') ? '' : 's') . "</strong>\n";

			}

			//find which are network-activated
			$network_plugins = array_flip( get_site_option('active_sitewide_plugins') );
			//remove those from our list
			$remove_network = array_diff( $plugins_list, $network_plugins );

			//show which not-network-activated plugins have 0 users
			echo "\nInactive (plugins with zero (0) users): \n";
			foreach( $remove_network as $k => $inactive ) {
				$realname = $all_plugins[$inactive]['Name'] . ' v' . $all_plugins[$inactive]['Version'];
				echo "\t{$realname}\n";
			}

		echo '</pre>';

		echo '<br />';
		echo "<pre style='background:#ccf'>\n";
			foreach( $pi as $siteid => $list ) {
				$details = get_blog_details( $siteid );

				$plugins = "$details->siteurl/wp-admin/plugins.php";
				$dash = "$details->siteurl/wp-admin/";
				$view = "$details->siteurl";
				$edit = network_admin_url( "site-info.php?id=$siteid" );

				echo "$details->blogname <a href='$edit'>($siteid) Edit</a> [<a href='$view'>View</a>] [<a href='$dash'>Dashboard</a>] [<a href='$plugins'>Plugins page</a>] \n";

				$tagged = array();
				$nottagged = '';
				foreach( $list as $name ) {
					//$realname = $all_plugins[$name]['Name'] . ' v' . $all_plugins[$name]['Version'];
					$realname = $all_plugins[$name]['Name'];
					if ( is_numeric( $tags[ $name ] ) )
						$nottagged .= "\t{$realname}\n";
					else
						$tagged["\t({$tags[ $name ]}) $realname\n"] = $tags[ $name ];
				}
				asort( $tagged );
				$tagged = array_keys( $tagged );
				echo implode( $tagged ) . $nottagged . "\n";

				unset( $tagged );
				unset( $nottagged );
			}

		echo '</pre>';

		echo '</div>';

	}// end page()

}
new activeplugins();